#!/bin/bash

commands=(
'date'
'uptime'
"grep -Ev '^[[:space:]]*(#.*)?$' /opt/monitor/op5/merlin/merlin.conf"
'mon node show'
'mon node status'
'mon node info'
'mon check distribution'
"printf '#merlin expired\0' | unixcat /opt/monitor/var/rw/nagios.qh"
'mon query ls status -c execute_host_checks,execute_service_checks,host_checks,service_checks,num_hosts,num_services,program_start'
'mon query ls hosts -c check_period | sort | uniq -c'
'mon query ls services -c check_period | sort | uniq -c'
'mon query ls hosts -c check_command | sort | uniq -c'
'mon query ls services -c check_command | sort | uniq -c'
'mon query ls hosts -c active_checks_enabled | sort | uniq -c'
'mon query ls services -c active_checks_enabled | sort | uniq -c'
'mon query ls hosts -c check_interval | sort | awk "{print \$1*1}" | uniq -c'
'mon query ls services -c check_interval | sort | awk "{print \$1*1}" | uniq -c'
'mon query ls services -c last_check | sort -n | sed "s/^/@/" | date -f - "+%Y-%m-%d %H:%M:%S %z"'
'mon query ls hosts -c last_check | sort -n | sed "s/^/@/" | date -f - "+%Y-%m-%d %H:%M:%S %z"'
'mon query ls services -c next_check | sort -n | sed "s/^/@/" | date -f - "+%Y-%m-%d %H:%M:%S %z"'
'mon query ls hosts -c next_check | sort -n | sed "s/^/@/" | date -f - "+%Y-%m-%d %H:%M:%S %z"'
'mon query ls hosts -c next_check,last_check last_check -ne 0 | awk -F";" "{print \$1-\$2}" | sort -n'
'mon query ls services -c next_check,last_check last_check -ne 0 | awk -F";" "{print \$1-\$2}" | sort -n'
"egrep -c '^define (host|service)dependency' /opt/monitor/var/objects.cache"
"sed -nr '/^define timeperiod .$/{:a;N;/\n\t+}$/{p;d};ba}' /opt/monitor/var/objects.cache"
)

node="$HOSTNAME"


die()
{
  printf 'ERROR: %s\n' "$1"
  exit 1
}

msg()
{
  printf '[%s] %s\n' "$(date '+%H:%M:%S')" "$1" 1>&2
}

noderun()
{
  local cmd numrunpad
  cmd="$1"

  printf -v numrunpad '%02d' "$numrun"
  d="$wdir/$numrunpad"
  mkdir "$d" || die "Unable to create dir: $d"

  printf '%s\n' "$cmd" > "$d/.cmd"
  msg "($numrunpad) Running command: $cmd"

  for node in $nodes; do
    msg "($numrunpad) @ $node .."
    mon node rawctrl "$node" "$cmd" &> "$d/$node.out"
    printf '%s\n' "$?" > "$d/$node.ret"
  done

  let numrun++
}

nodetest()
{
  local node
  for node in $nodes; do
    mon node rawctrl "$node" --test || exit $?
  done
}

mkwd()
{
  tdir="$(mktemp -d)" || die 'Unable to create tmpdir.'
  cd "$tdir" || die 'Unable to change to tmpdir.'

  ts="$(date +%s)"
  wname="analyze_distribution_${ts}_$HOSTNAME"
  wdir="$tdir/$wname"

  mkdir "$wdir" || die "Unable to mkdir workdir ($wdir)."
}

rmwd()
{
  rm -r "$wname"
  cd /
  rmdir "$tdir"
}

# set sane defaults
IFS=$' \t\n'
export LANG=C
export LC_NUMERIC=C
export LC_TIME=C

[ "$#" -ge '1' ] && die "syntax: mon sop analyze-distribution"

[ -n "$(mon node list --type=master)" ] && \
  die "This tool should be run on a master node, not a poller."

numrun=0
nodes="$HOSTNAME $(mon node list)"

msg 'Verifying node command execution capabilities...'
nodetest
printf '\n'

mkwd
msg "Created work directory: $wdir"
trap rmwd EXIT
printf '\n'

for cmd in "${commands[@]}"; do
  noderun "$cmd"
  printf '\n'
done

msg "Creating tarball."
tarball="$HOME/$wname.tar.gz"
tar cvvfz "$tarball" "$wname" || \
  die "Failed creating tarball: $tarball"

printf '\n\nCreated tarball: %s\n' "$tarball"
